#include <algorithm>
#include <vector>

#include "qthing/led_strip.hpp"
#include "qthing/qthing_legacy.hpp"

// void addUDPPacketCallback(std::string magicString, udpPacketCallback
// callback);

using namespace qthing;
using namespace std;

static vector<led_color_t>* active;
static vector<led_color_t>* inactive;
static vector<led_color_t>* third;

static led_color_handler_t local_led_color_handler;

static int activity = 0;
static float fade_level = 0;

static qthing::led_color_t color_handler(uint8_t strip_id, uint16_t address, qthing::led_color_t last_color) {
  // TODO: check if active buffer contains address

  if (strip_id == 0 && address == 0) {
    activity -= 1;

    if (activity <= 0) {
      // network not active
      activity = 0;

      fade_level -= 0.05f;
      if (fade_level < 0) {
        fade_level = 0;
      }
    } else {
      // network active

      fade_level += 0.05f;
      if (fade_level > 1) {
        fade_level = 1;
      }
    }
  }

  led_color_t color = active->at(address + strip_id * 295);
  if (color.r > 0) {
    color.r -= 1;
  }
  if (color.g > 0) {
    color.g -= 1;
  }
  if (color.b > 0) {
    color.b -= 1;
  }
  active->at(address + strip_id * 295) = color;

  if (fade_level >= 1) {
    return color;
  } else {
    return color * fade_level + local_led_color_handler(strip_id, address, last_color) * (1 - fade_level);
  }
}

static void udp_packet_handler(udpPacket udp_packet) {
  size_t header_length = 4;
  size_t length = udp_packet.length;
  const char* payload = udp_packet.payload;

  if (payload[0] == 'c') {
    header_length = 8;
  }

  payload += header_length;
  length -= header_length;

  // command has been parsed by udp handler too
  payload += 1;
  length -= 1;

  uint8_t flush = *((uint8_t*)payload);
  payload += 1;
  length -= 1;

  uint32_t offset = *((uint32_t*)payload);
  payload += 4;
  length -= 4;

  if (length / sizeof(led_color_t) <= inactive->size() - offset) {
    // copy active to inactive to accumulate packets
    // copy (active->begin(), active->end(), inactive->begin());

    copy((led_color_t*)payload, ((led_color_t*)payload) + (length / sizeof(led_color_t)), inactive->begin() + offset);

    // TODO: fragment number

    // TODO: sequence number

    if (flush) {
      auto temp = active;
      active = inactive;
      inactive = third;
      third = temp;

      activity = 240;

      fill(inactive->begin(), inactive->end(), led_color_t());
    }
  }
}

namespace qthing {
void configure_network_color_handler() {
  active = new vector<led_color_t>(1475);
  inactive = new vector<led_color_t>(1475);
  third = new vector<led_color_t>(1475);

  local_led_color_handler = get_color_handler();
  set_color_handler(color_handler);

  addUDPPacketCallback(
      "qctl"
      "c",
      udp_packet_handler);
  addUDPPacketCallback("congress", udp_packet_handler);
}
}  // namespace qthing
