#include "qthing/rc_433.hpp"

#include <RCSwitch.h>

#include <string>

#include "Arduino.h"
#include "esp_log.h"
#include "qthing/mqtt.hpp"

#define RC_TAG "RC_433"

namespace qthing {

RCSwitch mySwitch = RCSwitch();

bool send_enabled = false;
bool receive_enabled = false;

void rc_switch_on(const std::string& code) {
  const char* s_code = code.c_str();
  ESP_LOGI(RC_TAG, "Sending %s on", s_code);
  mySwitch.switchOn(s_code);
}

void rc_switch_off(const std::string& code) {
  const char* s_code = code.c_str();
  ESP_LOGI(RC_TAG, "Sending %s off", s_code);
  mySwitch.switchOff(s_code);
}

void rc_switch(const std::string& code, bool state) {
  if (state) {
    rc_switch_on(code);
  } else {
    rc_switch_off(code);
  }
}

void rc_receive_task(void* arg) {
  while (true) {
    if (mySwitch.available()) {
      ESP_LOGI(RC_TAG, "Received: %lu, bit length: %u, protocol: %u", mySwitch.getReceivedValue(),
               mySwitch.getReceivedBitlength(), mySwitch.getReceivedProtocol());
      mySwitch.resetAvailable();
    }
    vTaskDelay(50 / portTICK_PERIOD_MS);
  }
}

void rc_enable_send(const std::string& send_on_topic, const std::string& send_off_topic, gpio_num_t gpio) {
  if (send_enabled) {
    ESP_LOGE(RC_TAG, "433MHz sending is already enabled;");
    return;
  }
  send_enabled = true;
  mySwitch.enableTransmit(gpio);
  add_message_callback(send_on_topic, [](const std::string& code) { rc_switch_on(code); });
  add_message_callback(send_off_topic, [](const std::string& code) { rc_switch_off(code); });
}

void rc_enable_receive(const std::string& topic, gpio_num_t gpio) {
  if (receive_enabled) {
    ESP_LOGE(RC_TAG, "433MHz receiving is already enabled");
    return;
  }
  mySwitch.enableReceive(gpio);
  xTaskCreate(rc_receive_task, "rc_receive_task", 2048, NULL, 3, NULL);
}

}  // namespace qthing
